@@ -8,7 +8,8 @@ from torch import nn
 from torch.nn import functional as F
 
 from .computation_node import ModuleNode, FunctionNode, TensorNode, NodeContainer
-from .computation_graph import ComputationGraph
+
+from .utils import OrderedSet
 
 # Needed for module wrapper and resetting
 _orig_module_forward = torch.nn.Module.__call__
@@ -75,9 +76,7 @@ def creation_ops_wrapper(_orig_op: Callable[..., Any]) -> Callable[..., Any]:
     return _func
 
 
-def module_forward_wrapper(
-    model_graph: ComputationGraph,
-) -> Callable[..., Any]:
+def module_forward_wrapper() -> Callable[..., Any]:
     '''Wrapper for forward functions of modules'''
     def _module_forward_wrapper(mod: nn.Module, *args: Any, **kwargs: Any) -> Any:
         '''Forward prop of module for RecorderTensor subclass
@@ -105,17 +104,14 @@ def module_forward_wrapper(
         input_context.append({cur_node: []})
         for node in input_nodes:
             node.add_outputs(cur_node)
-            node.name = "hidden-tensor"
-        cur_node.set_input_shape(
-            reduce_data_info([args, kwargs], collect_shape, [])
-        )
 
         tensor_to_node: dict[RecorderTensor, TensorNode] = (
             reduce_data_info([args, kwargs], collect_tensor_node_id_dict, {})
         )
         attach_kwargs = {
-            'inputs': cur_node, 'depth': cur_node.depth+1,
+            'inputs': cur_node, 'depth': cur_depth+1,
             'context': input_context[-1][cur_node], 'is_aux': True,
+            'name': 'auxiliary-tensor'
         }
 
         traverse_data_inplace(
@@ -126,14 +122,16 @@ def module_forward_wrapper(
         # this seems not to be necessary so far
         out = _orig_module_forward(mod, *args, **kwargs)
 
-        traverse_data_inplace(
-            out, change_depth_name(-1, 'output-tensor', cur_node)
+        # pop appropriate nodes, see implementation below
+        output_recorder: OrderedSet[RecorderTensor] = (
+            reduce_data_info(out, collect_tensor, OrderedSet())
         )
 
-        # pop appropriate nodes, see implementation below
-        output_recorder: list[RecorderTensor] = (
-            reduce_data_info(out, collect_tensor, [])
+        traverse_data_inplace(
+            output_recorder,
+            process_output_node(cur_node)
         )
+
         traverse_data_inplace(
             [args, kwargs], pop_after_forward, recorded_output=output_recorder
         )
@@ -146,29 +144,10 @@ def module_forward_wrapper(
         for output_node in output_nodes:
             cur_node.end_nodes.add(output_node)
             output_node.context = input_context
-            # output of empty modules
-            if output_node.depth == cur_node.depth:
-                output_node.node_id = str(id(output_node))
-
-        if model_graph.hide_module_functions and cur_node.is_container:
-            input_context.pop()
-            input_context.append(cur_node)
-            for out_node in output_nodes:
-                out_node.context = input_context
-                input_context.append(out_node)
-                out_node.input_hierarchy = {
-                    key_word: value
-                    for key_word, value in out_node.input_hierarchy.items()
-                    if key_word != cur_node.depth+1
-                }
-            # keep removing until all output tensor nodes
-            # are outputs of current module node
-            while not (
-                    reduce_data_info(out, collect_tensor_node, NodeContainer())
-                    .issubset(cur_node.outputs)  # type: ignore[arg-type]
-            ):
-                remove_func_module(out, cur_node)
 
+        cur_node.set_input_shape(
+            reduce_data_info([args, kwargs], collect_shape, [])
+        )
         cur_node.set_output_shape(reduce_data_info(out, collect_shape, []))
         return out
 
@@ -242,7 +221,7 @@ class RecorderTensor(torch.Tensor):
         # dont create any node, give the result only
         if not args_nodes:
             return out
-        if not reduce_data_info(out, collect_tensor, []):
+        if not reduce_data_info(out, collect_tensor, OrderedSet()):
             return out
 
         # Create function_node and connect to its inputs tensor node
@@ -254,11 +233,12 @@ class RecorderTensor(torch.Tensor):
 
         for i in args_nodes:
             i.add_outputs(cur_node)
-            i.name = 'hidden-tensor'
+
         input_context.append(cur_node)
         attach_kwargs = {
-            'inputs': cur_node, 'depth': cur_node.depth, "context": input_context,
+            'inputs': cur_node, 'depth': cur_depth, "context": input_context,
             'is_aux': False, 'input_hierarchy': {cur_depth: cur_node},
+            'name': 'output-tensor' if cur_depth == 0 else 'hidden-tensor'
         }
         traverse_data_inplace(out, attach_node(attach_kwargs))
 
@@ -272,13 +252,6 @@ class RecorderTensor(torch.Tensor):
         )
         cur_node.set_output_shape(reduce_data_info(out, collect_shape, []))
 
-        out_nodes: NodeContainer[TensorNode] = (
-            reduce_data_info(out, collect_tensor_node, NodeContainer())
-        )
-
-        for out_node in out_nodes:
-            assert out_node.context is input_context
-
         return out
 
 
@@ -365,7 +338,7 @@ def attach_node(
 
 def pop_after_forward(
     r_in: RecorderTensor,
-    recorded_output: NodeContainer[RecorderTensor]
+    recorded_output: OrderedSet[RecorderTensor]
 ) -> None:
     '''Removes/pops nodes from RecorderTensors to maintain correct nodes
     Two types of process exist for types of modules:
@@ -376,7 +349,7 @@ def pop_after_forward(
     in_place_func_message = (
         'Tensor before and after inplace operation must have the same memory address'
     )
-    output_id: NodeContainer[int] = NodeContainer(id(x) for x in recorded_output)
+    output_id: OrderedSet[int] = OrderedSet(id(x) for x in recorded_output)
 
     if not id(r_in) in output_id:
         _ = reduce_data_info(
@@ -395,28 +368,6 @@ def pop_after_forward(
         r_in.tensor_nodes.pop(-2)
 
 
-def remove_func_module(out: Any, mod_node: ModuleNode,) -> None:
-    '''Removes all input nodes of RecorderTensor Nodes
-    from the graph
-    '''
-    my_col: NodeContainer[TensorNode] = NodeContainer()
-
-    output_tensor_nodes: NodeContainer[TensorNode] = (
-        reduce_data_info(out, collect_tensor_node, NodeContainer())
-    )
-    # collect all input nodes
-    for out_node in output_tensor_nodes:
-        for in_node in out_node.inputs:
-            my_col.add(in_node)  # type: ignore[arg-type]
-
-    for par in my_col:
-        assert not isinstance(par, ModuleNode), (
-            f'Module Node {mod_node} has no outputs module,'
-            f'hence must not have node of Module Node type {par}'
-        )
-        par.remove()
-
-
 def collect_tensor_node(
     recorded_data: RecorderTensor,
     collected: NodeContainer[TensorNode],
@@ -438,9 +389,9 @@ def collect_tensor_node_id_dict(
 
 
 def collect_tensor(
-    recorded_data: RecorderTensor, collected: list[RecorderTensor]
+    recorded_data: RecorderTensor, collected: OrderedSet[RecorderTensor]
 ) -> None:
-    collected.append(recorded_data)
+    collected.add(recorded_data)
 
 
 def collect_shape(
@@ -449,12 +400,34 @@ def collect_shape(
     collected.append(tuple(recorded_data.shape))
 
 
-def change_depth_name(
-    depth_delta: int, name: str, cur_node: ModuleNode
+def process_output_node(
+    cur_node: ModuleNode
 ) -> Callable[..., Any]:
     def _func(recorded_data: RecorderTensor) -> None:
-        recorded_data.tensor_nodes[-1].depth += depth_delta
+        output_node = recorded_data.tensor_nodes[-1]
+        cur_depth = cur_node.depth
+        # if output node is reused inside module or empty module is used
+        # introduce node for empty pass function
+        if not output_node.is_main_output() or output_node.is_aux:
+            out_pass = FunctionNode(
+                lambda x: x, output_node.depth, output_node,
+                name='empty-pass'
+            )
+            output_node.add_outputs(out_pass)
+            output_node.context.append(out_pass)
+
+            recorded_data.tensor_nodes[-1] = TensorNode(
+                recorded_data, output_node.depth, out_pass,
+                context=output_node.context, is_aux=False,
+                input_hierarchy={
+                    recorded_data.tensor_nodes[-1].depth: out_pass
+                }
+            )
+            out_pass.add_outputs(recorded_data.tensor_nodes[-1])
+            output_node.context.append(recorded_data.tensor_nodes[-1])
+
+        recorded_data.tensor_nodes[-1].depth = cur_depth
+        name = 'output-tensor' if cur_depth == 0 else 'hidden-tensor'
         recorded_data.tensor_nodes[-1].name = name
-        cur_depth = recorded_data.tensor_nodes[-1].depth
         recorded_data.tensor_nodes[-1].input_hierarchy[cur_depth] = cur_node
     return _func
