@@ -177,12 +177,13 @@ def draw_graph(
     # seems to work visually well
     node_attr = {
         'style': 'filled',
-        'shape': 'record',
+        'shape': 'plaintext',
         'align': 'left',
         'fontsize': '10',
         'ranksep': '0.1',
         'height': '0.2',
         'fontname': 'Linux libertine',
+        'margin': '0',
     }
 
     edge_attr = {
@@ -194,8 +195,8 @@ def draw_graph(
         directory=directory, filename=filename
     )
 
-    input_recorder_tensor, input_nodes = process_input(
-        input_data, input_size, device, dtypes
+    input_recorder_tensor, kwargs_record_tensor, input_nodes = process_input(
+        input_data, input_size, kwargs, device, dtypes
     )
 
     model_graph = ComputationGraph(
@@ -205,7 +206,7 @@ def draw_graph(
 
     forward_prop(
         model, input_recorder_tensor, device,
-        model_mode, model_graph, **kwargs
+        model_mode, **kwargs_record_tensor
     )
 
     model_graph.fill_visual_graph()
@@ -220,7 +221,6 @@ def forward_prop(
     x: CORRECTED_INPUT_DATA_TYPE,
     device: torch.device | str,
     mode: str,
-    model_graph: ComputationGraph,
     **kwargs: Any,
 ) -> None:
     '''Performs forward propagation of model on RecorderTensor
@@ -235,7 +235,7 @@ def forward_prop(
             raise RuntimeError(
                 f"Specified model mode not recognized: {mode}"
             )
-        new_module_forward = module_forward_wrapper(model_graph)
+        new_module_forward = module_forward_wrapper()
         with Recorder(_orig_module_forward, new_module_forward):
             with torch.no_grad():
                 if isinstance(x, (list, tuple)):
@@ -257,12 +257,14 @@ def forward_prop(
 def process_input(
     input_data: INPUT_DATA_TYPE | None,
     input_size: INPUT_SIZE_TYPE | None,
+    kwargs: Any,
     device: torch.device | str,
     dtypes: list[torch.dtype] | None = None,
-) -> tuple[CORRECTED_INPUT_DATA_TYPE, NodeContainer[TensorNode]]:
+) -> tuple[CORRECTED_INPUT_DATA_TYPE, Any, NodeContainer[TensorNode]]:
     """Reads sample input data to get the input size."""
     x = None
     correct_input_size = []
+    kwargs_recorder_tensor = traverse_data(kwargs, get_recorder_tensor, type)
     if input_data is not None:
         x = set_device(input_data, device)
         x = traverse_data(x, get_recorder_tensor, type)
@@ -276,9 +278,11 @@ def process_input(
         x = get_input_tensor(correct_input_size, dtypes, device)
 
     input_data_node: NodeContainer[TensorNode] = (
-        reduce_data_info(x, collect_tensor_node, NodeContainer())
+        reduce_data_info(
+            [x, kwargs_recorder_tensor], collect_tensor_node, NodeContainer()
+        )
     )
-    return x, input_data_node
+    return x, kwargs_recorder_tensor, input_data_node
 
 
 def validate_user_params(
@@ -366,7 +370,7 @@ def get_recorder_tensor(
     input_node = TensorNode(
         tensor=input_recorder_tensor,
         depth=0,
-        name='input_node',
+        name='input-tensor',
     )
 
     input_recorder_tensor.tensor_nodes.append(input_node)
